#!/bin/bash
# ----------------------------------------------------------------------------
# For details on this code see the books "z/VM and Linux on IBM System z:
# The Virtualization Cookbook for <x>" on the Web at: http://linuxvm.org/present
# where <x> is "SLES 10" or "RHEL 5"
#
# This code is shipped under the Common Public License. See the file
# Common-Public-License.txt or http://www.opensource.org/licenses/cpl1.0.txt
# ----------------------------------------------------------------------------
#
# backupVM.sh  - do a backup of z/VM files by linking to important disks:
#  MAINT 191 - USER DISKMAP
#  MAINT 2CC - USER DIRECT
#  MAINT CF1 - SYSTEM CONFIG
#  TCPMAINT 198 - <systemID> TCPIP, SYSTEM DTCPARMS
#  TCPMAINT 592 - TCPIP DATA
#  AUTOLOG1 191 - PROFILE EXEC
#  LNXMAINT 192 - * PARMFILE, * PARM-S10, SLES* *
# The system that this is run on should have the LNKNOPAS directory option so
# all the above minidisks can be linked via vmcp with no password necessary 
#
#+--------------------------------------------------------------------------+
function cpCmd()
# echo a CP command and invoke it via the vmcp module/command
#   Arg 1-n: the command to issue
#   Return: the command's return code
#+--------------------------------------------------------------------------+
 {
# parse output to get return code: awk -F# splits line at '#' with rc at end
  output=`vmcp $@ 2>&1`
  ret_val=0
  ret_val=`echo $output | grep "Error: non-zero CP response" | awk -F# '{print $2}'`
  return $ret_val
 }

#+--------------------------------------------------------------------------+
function linkAccess()
# link and access a minidisk 
#   Arg 1: the source disk user ID 
#   Arg 2: the source disk virtual device number
#   Arg 3: placeholder for code readability 
#   Arg 4: the virtual device number to link the minidisk as
#+--------------------------------------------------------------------------+
 {
  userID=$1
  vaddr=$2
  linkVaddr=$4
  cpCmd QUERY VIRTUAL $linkVaddr 
  queryRC=$?
  if [ $queryRC = 40 ]; then # linkVaddr is free
    cpCmd LINK $userID $vaddr $linkVaddr RR 
    rc=$?
    if [ $rc != 0 ]; then # can't link - punt
      echo "Error: unable to link $userID $vaddr read-only"
      echo "LINK $userID $vaddr $linkVaddr RR returned $rc"
      exit
    fi
  else # linkVaddr is not free 
    echo "Error: Virtual address $linkVaddr is not free?"
    echo "QUERY VIRTUAL $linkVaddr returned $queryRC"
  fi
  # activate new disk then sleep 1 second for CP to catch up
  chccwdev -e $linkVaddr > /dev/null 2>&1
  sleep 1
 }

#+--------------------------------------------------------------------------+
function getFile()
# get a file on a CMS disk 
#   Arg 1: the file or file pattern to get 
#   Arg 2: placeholder for code readability 
#   Arg 3: the virtual device number to detach 
#+--------------------------------------------------------------------------+
 {
  file=$1
  vaddr=$3
  CMSdevice=`cat /proc/dasd/devices | grep $vaddr | awk '{print $7}'`
  cmsfscat -a -d /dev/$CMSdevice $file > $file 
  rc=$?
  if [ $rc != 0 ]; then
    echo "Error: cmsfscat returned $rc backing up $file"
  else
    echo "Backing up $file seems to have succeeded"
  fi  
 }

#+--------------------------------------------------------------------------+
function getFiles()
# get all files on a CMS disk 
#   Arg 1: the virtual device number to copy files from 
#+--------------------------------------------------------------------------+
 {
  vaddr=$1
  CMSdevice=`cat /proc/dasd/devices | grep $vaddr | awk '{print $7}'`
  let i=0
  cmsfslst -d /dev/$CMSdevice | while read oneline
  do # first record is headers, second is "DIRECTOR", third is "ALLOCMAP"
    if (($i > 2)); then # we are into the CMS files
      fn=`echo $oneline | awk '{print $1}'`
      ft=`echo $oneline | awk '{print $2}'`
      cmsfscat -a -d /dev/$CMSdevice $fn.$ft > $fn.$ft 
      rc=$?
      if [ $rc != 0 ]; then # error
        echo "Error: cmsfscat returned $rc backing up $fn.$ft"
      else
        echo "Backing up $fn.$ft seems to have succeeded"
      fi
    fi
    let i=$i+1
  done 
 }

#+--------------------------------------------------------------------------+
function relDetach()
# link and access a minidisk
#   Arg 1: the virtual device number to detach 
#   Return: the command's return code
#+--------------------------------------------------------------------------+
 {
  chccwdev -d $1 > /dev/null 2>&1
  cpCmd DET $1 
 }

# main()
rm /backup/vm/*  # erase all files so old files are left around
cd /backup/vm

# run the getVMinfo script to save some basic VM information
getVMinfo.sh > zVMinfo.txt

# get USER DISKMAP from MAINT 191
linkAccess MAINT 191 as 1191
getFile USER.DISKMAP from 1191
relDetach 1191

# get USER DIRECT from MAINT 2CC
linkAccess MAINT 2CC as 12cc 
getFile USER.DIRECT from 12cc 
relDetach 12cc

# get SYSTEM CONFIG from MAINT CF1
linkAccess MAINT CF1 as 1cf1
getFile SYSTEM.CONFIG from 1cf1
relDetach 1cf1

# get TCPIP files from TCPMAINT 198
systemID=`vmcp q userid | awk '{print $3}'`
linkAccess TCPMAINT 198 as 1198
getFile $systemID.TCPIP from 1198
getFile SYSTEM.DTCPARMS from 1198
relDetach 1198

# get TCPIP DATA from TCPMAINT 592
linkAccess TCPMAINT 592 as 1592
getFile TCPIP.DATA from 1592
relDetach 1592

# get PROFILE EXEC from AUTOLOG1 191 - rename to different FT
linkAccess AUTOLOG1 191 as 2191
CMSdevice=`cat /proc/dasd/devices | grep 2191 | awk '{print $7}'`
cmsfscat -a -d /dev/$CMSdevice PROFILE.EXEC > PROFILE.EXECAUT1 
relDetach 2191 

# Get all files from LNXMAINT 192 which is our 191 disk
getFiles 191
