#!/bin/bash
#
# clone.sh <LinuxUserID> - clone a Linux server running under z/VM
#
# For details on how this script works see the book: 
#   "z/VM and Linux on IBM System z: The Virtualization Cookbook"
#   on the Web at: http://linuxvm.org/present/misc/virt-cookbook-1.pdf  
#
# ----------------------------------------------------------------------------
# THE PROGRAM IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
# OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION, ANY
# WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY
# OR FITNESS FOR A PARTICULAR PURPOSE.
# NEITHER RECIPIENT NOR ANY CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING WITHOUT LIMITATION LOST PROFITS), HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OR
# DISTRIBUTION OF THE PROGRAM OR THE EXERCISE OF ANY RIGHTS GRANTED
# HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES
# ----------------------------------------------------------------------------
 
#+--------------------------------------------------------------------------+
function help()
# give help 
#+--------------------------------------------------------------------------+
 {
  echo "Usage: clone [options] target_linux_ID"
  echo ""
  echo "  Clone Linux system from minidisk $sdisk to target_linux_ID"
  echo "  options:"
  echo "    -v or --verbose: verbose"
  exit 1
 }

#+--------------------------------------------------------------------------+
function process_arguments() 
# give help 
#+--------------------------------------------------------------------------+
 {
  verbose="off"
  if [ $# = 0 ]; then # user did not pass an argument
    echo "error: at least one argument is needed"
    echo ""
    help 
  fi
  for arg in $*
  do
    case $arg in
      -v|--verbose) 
         verbose="on"
         ;;
      *)
        clone_id=`echo $arg | tr '[a-z]' '[A-Z]'`     # fold target user ID to upper case
        ;;
    esac
  done
  if [ $clone_id = "none" ]; then # new linux user ID was not passed 
    echo "Error: new Linux user ID must be an argument"
    echo ""
    help 
  fi
 }

#+--------------------------------------------------------------------------+
function cp_cmd()
# echo a CP command and invoke it via the vmcp module/command
#   Arg1-n: the command to issue 
#   Return: the command's return code
#+--------------------------------------------------------------------------+
 {
  echo "Invoking CP command: $@"
# parse output to get return code: awk -F# splits line at '#' with rc at end
  output=`vmcp $@ 2>&1`
  echo "$output"
  ret_val=0
  ret_val=`echo $output | grep "Error: non-zero CP response" | awk -F# '{print $2}'`
  return $ret_val
 }

#+--------------------------------------------------------------------------+
function check_target_id()
# Verify user ID exists and is logged off 
#+--------------------------------------------------------------------------+
 {
  cp_cmd QUERY $clone_id
  case $? in
    0)  # user ID is logged on or disconnected 
      echo "$clone_id user ID must be logged off"
      exit 2
      ;;
    3)  # user ID does not exist
      echo "$clone_id user ID does not exist"
      exit 3
      ;;
   45) # user ID is logged off - this is correct
      ;;
   *) # unexpected
      echo "$clone_id user ID must exist and be logged off"
      exit 4
  esac
 }

#+--------------------------------------------------------------------------+
function prepare_ipaddr()
# Prepare an IP address by adding a backslash before any "."s 
# to make it 4 digits
#    Arg 1: The IP address to be modified
# Return:
#    The new value is written to the global variable new_ipaddr
#+--------------------------------------------------------------------------+
 {
  new_ipaddr=`echo $1 | sed -e 's:\.:\\\.:g'`
 }

#+--------------------------------------------------------------------------+
function prepare_vaddr()
# Prepare a z/VM virtual address by folding to lower case and prepending leading zeros
# to make it 4 digits
#    Arg 1: The vaddr to be modified
# Return:
#    The new value is written to the global variable new_vaddr
#+--------------------------------------------------------------------------+
 {
  new_vaddr=`echo $1 | tr '[A-Z]' '[a-z]'`  # fold to lower case
  let leadingZeros=4-${#1}                  # determine number of zeros to add
  let i=0
  while [ $i -lt $leadingZeros ]; do
    new_vaddr="0$new_vaddr"
    i=$[$i+1]
  done
 }

#+--------------------------------------------------------------------------+
function copy_disk()
# Try to use z/VM FLASHCOPY to copy one disk to another. 
#   Arg 1: Source minidisk vaddr
#   Arg 2: the word "to"
#   Arg 3: Target user ID 
#   Arg 4: Target virtual address
# Return code
#   0: success with FLASHCOPY
#   1: success with dasdfmt and dd
#   2: Target vaddr already in use?
#   3: CP LINK failed
#   4: unable to enable target minidisk 
#   5: can't find source disk in /dev/dasd/devices 
#   6: can't find target disk in /dev/dasd/devices 
#   7: dasdfmt failed
#   8: dd failed
#+--------------------------------------------------------------------------+
 {
  ret_val=0
  source_mdisk=$1
  target_userid=$3
  target_vaddr=$4
  cp_cmd QUERY VIRTUAL $target_vaddr  
  rc=$?
  if [ $rc != 40 ]; then 
    echo "Error: CP QUERY VIRTUAL $target_vaddr failed with $rc"
    echo "The target device address is already in use"
    return 2 
  fi
  cp_cmd LINK $target_userid $source_mdisk $target_vaddr MR
  rc=$?
  if [ $rc != 0 ]; then # LINK failed
    echo "Error: CP LINK $target_userid $source_mdisk $target_vaddr failed with $rc"
    return 3
  fi
  cp_cmd FLASHCOPY $source_mdisk 0 END $target_vaddr 0 END 
  rc=$?
  if [ $rc != 0 ]; then # FLASHCOPY failed
    echo "FLASHCOPY $source_mdisk $target_vaddr failed with $rc"
    echo "Falling back to dasdfmt and dd copy"
#   enable target disk
    sleep 1
    chccwdev -e $target_vaddr
    rc=$?
    if [ $rc != 0 ]; then # unable to enable target disk
      echo "Error: unable to enable $target_disk, rc from chccwdev = $rc"
      cp_cmd DETACH $target_vaddr
      return 4
    fi
#   get device name of source disk
    source_dev=`cat /proc/dasd/devices | grep "$source_mdisk(ECKD)" | awk '{ print $7 }'`
    if [ "$source_dev" = "" ]; then
      cat /proc/dasd/devices
      echo "Error: can't find $source_mdisk(ECKD) in /proc/dasd/devices"
      chccwdev -d $target_vaddr # clean up
      cp_cmd DETACH $target_vaddr
      return 5
    fi
#   get device name of target disk
    target_dev=`cat /proc/dasd/devices | grep "$target_vaddr(ECKD)" | awk '{ print $7 }'`
    if [ "$target_dev" = "" ]; then
      cat /proc/dasd/devices
      echo "Error: can't find $target_vaddr(ECKD) in /proc/dasd/devices"
      chccwdev -d $target_vaddr # clean up
      cp_cmd DETACH $target_vaddr
      return 6 
    fi
#   dasdfmt target disk
    echo "Invoking command: dasdfmt -b 4096 -y -f /dev/$target_dev"
    dasdfmt -b 4096 -y -f /dev/$target_dev
    rc=$?
    if [ $rc != 0 ]; then # dasdfmt failed
      echo "Error: dasdfmt -b 4096 -y -f /dev/$target_dev failed with $rc"
      chccwdev -d $target_vaddr # clean up
      cp_cmd DETACH $target_vaddr
      return 7
    fi
#   copy from source disk to target via dd
    echo "Invoking command: dd bs=4096 if=/dev/$source_dev of=/dev/$target_dev"
    dd bs=4096 if=/dev/$source_dev of=/dev/$target_dev
    rc=$?
    if [ $rc != 0 ]; then # dd failed
      echo "Error: dd bs=4096 if=/dev/$source_dev of=/dev/$target_dev failed with $rc"
      chccwdev -d $target_vaddr # clean up
      cp_cmd DETACH $target_vaddr
      return 8
    fi
    chccwdev -d $target_vaddr
    echo "Copying disk via dasdfmt/dd succeeded ..."
    ret_val=1 # success with dasdfmt and dd
  fi # if FLASHCOPY failed
  sync # sync disks
  cp_cmd DETACH $target_vaddr
  echo "Copying disk via FLASHCOPY succeeded ..."
  return $ret_val 
 } # copy_disk()

#+--------------------------------------------------------------------------+
function get_conffile_info()
# Bring 191 minidisk online to be read by cmsfs and check for two CONF-RH5 files
#+--------------------------------------------------------------------------+
 {
  # recycle 191 to pick up latest changes
  chccwdev -d 191
  chccwdev -e 191
#  sleep 1 # timing problem?
  rc=$?
  if [ $rc != 0 ]; then # unable to enable 191 disk
    echo "unable to enable 191, rc from chccwdev = $rc"
    exit 5
  fi
  CMSdisk=`lsdasd | grep 0191 | awk '{ print $7 }'`
  cmsfslst -d /dev/$CMSdisk | grep -i $clone_id | grep CONF-RH5
  rc=$?
  if [ $rc != 0 ]; then
    echo "Error: $clone_id CONF-RH5 not found on 191 minidisk. Exiting" 
    exit 6
  fi

# get informaton about target
  export local $(cmsfscat -a -d /dev/$CMSdisk $clone_id.CONF-RH5)
  target_hostname=$HOSTNAME
  target_domain=${HOSTNAME#*.}
  target_host=${HOSTNAME%.$target_domain}
  target_IP=$IPADDR
  target_DNS=$DNS
  target_GW=$GATEWAY
  target_netmask=$NETMASK
  target_network=$NETWORK
  target_broadcast=$BROADCAST
  prepare_vaddr $SUBCHANNELS
  target_subchannels=$new_vaddr

# get information about source 
  source_guestID=`cat /proc/sysinfo | grep "VM00 Name" | awk '{ print $3 }'`
  export local $(cmsfscat -a -d /dev/$CMSdisk $source_guestID.CONF-RH5)
  source_hostname=$HOSTNAME
  source_domain=${HOSTNAME#*.}
  source_host=${HOSTNAME%.$source_domain}
  prepare_ipaddr $IPADDR
  source_IP=$new_ipaddr
  prepare_ipaddr $DNS
  source_DNS=$new_ipaddr
  prepare_ipaddr $GATEWAY
  source_GW=$new_ipaddr
  prepare_ipaddr $NETMASK
  source_netmask=$new_ipaddr
  prepare_ipaddr $NETWORK
  source_network=$new_ipaddr
  prepare_ipaddr $BROADCAST
  source_broadcast=$new_ipaddr
  prepare_vaddr $SUBCHANNELS
  source_subchannels=$new_vaddr
 }

#+--------------------------------------------------------------------------+
function ask_are_you_sure()
# Ask "Are you sure?" - if not, then exit 
#+--------------------------------------------------------------------------+
 {
  echo ""
  echo "WARNING!!: this will copy disk $sdisk to $clone_id 100"
  echo "New host name will be:  $target_hostname"
  echo "New TCP/IP address will be: $target_IP"
  echo "Other network data is retrieved from $clone_id CONF-RH5 on 191 disk"
  echo -n "Are you sure you want to overwrite these disks (y/n): "
  read ans
  if [ $ans != "y" ]; then
    exit 7
  fi 
  echo "Please enter a brief description of this server"
  read descr
 }

#+--------------------------------------------------------------------------+
function copy_system() 
# copy master image on $sdisk (default 100) to target user ID 
#+--------------------------------------------------------------------------+
 {
  echo "Copying $sdisk root file system to $clone_id ..."
  copy_disk $sdisk to $clone_id 1100 # copy source to target root file system
  rc=$?
  if [ $rc -gt 1 ]; then # both FLASHCOPY and dasdfmt/dd failed
    echo "Copying disk failed with $rc"
    exit 9
  fi
 }

#+--------------------------------------------------------------------------+
function modify_cloned_image() 
# Mount newly copied system over /mnt/rhel5cloned and modify networking info
#   Arg 1: target userid 
#   Arg 2: target minidisk 
#   Arg 3: target virtual device address 
# Return code
#   0: success 
#+--------------------------------------------------------------------------+
 {
  target_userid=$1
  target_mdisk=$2
  target_vaddr=$3
  echo "Mounting newly cloned image over $cloned_mnt_pt ..."
  cp_cmd LINK $target_userid $target_mdisk $target_vaddr MR
  rc=$?
  sleep 1
  if [ $rc != 0 ]; then # LINK failed
    echo "Fatal error: CP LINK $target_userid $target_mdisk $target_vaddr MR failed with $rc"
    cp_cmd DET $target_vaddr
    exit 10 
  fi
  if [ ! -d $cloned_mnt_pt ]; then 
    mkdir $cloned_mnt_pt
    rc=$?
    if [ $rc != 0 ]; then
      echo "Fatal Error: mkdir $cloned_mnt_pt failed with $rc"
      cp_cmd DET $target_vaddr
      exit 10
    fi
  fi
  chccwdev -e $target_vaddr 
  sleep 2
  rc=$?
  if [ $rc != 0 ]; then
    echo "Fatal error: chccwdev -e $target_vaddr failed with $rc"
    cp_cmd DET $target_vaddr
    exit 10
  fi
  cloned_DASD=`cat /proc/dasd/devices | grep "$target_vaddr(ECKD)" | awk '{ print $7 }'`
  if [ "$cloned_DASD" = "" ]; then
    cat /proc/dasd/devices
    echo "Fatal error: can't find $target_vaddr(ECKD) in /proc/dasd/devices"
    chccwdev -d $target_vaddr
    exit 10
  fi
  cloned_fs="/dev/${cloned_DASD}1"
  echo "Mounting $cloned_fs over $cloned_mnt_pt ..."
  mount $cloned_fs $cloned_mnt_pt
  rc=$?
  if [ $rc != 0 ]; then
    echo "Fatal error: mount $cloned_fs $cloned_mnt_pt failed with $rc"
    lsdasd
    cp_cmd DET $target_vaddr
    exit 10
  fi
  echo "Modifying cloned image under $cloned_mnt_pt ..."
  sed --in-place -e "s/$source_IP/$target_IP/g" \
    -e "s/$source_hostname/$target_hostname/g" \
    -e "s/$source_host/$target_host/g" $cloned_mnt_pt/etc/hosts
  sed --in-place -e "s/$source_hostname/$target_hostname/g" \
    -e "s/$source_GW/$target_GW/g" $cloned_mnt_pt/etc/sysconfig/network
  sed --in-place -e "s/$source_IP/$target_IP/g" \
    -e "s/$source_netmask/$target_netmask/g" \
    -e "s/$source_broadcast/$target_broadcast/g" \
    -e "s/$source_network/$target_network/g" \
    -e "s/$source_subchannels/$target_subchannels/g" \
    $cloned_mnt_pt/etc/sysconfig/network-scripts/ifcfg-eth0
  sed --in-place -e "s/$source_domain/$target_domain/g" \
    -e "s/$source_DNS/$target_DNS/g" $cloned_mnt_pt/etc/resolv.conf

# Regenerate SSH keys
  echo "Regenerating SSH keys in $cloned_mnt_pt/etc/ssh/ ..."
  rm $cloned_mnt_pt/etc/ssh/ssh_host*
  ssh-keygen -t rsa -N "" -q -f $cloned_mnt_pt/etc/ssh/ssh_host_rsa_key
  ssh-keygen -t dsa -N "" -q -f $cloned_mnt_pt/etc/ssh/ssh_host_dsa_key
  ssh-keygen -t rsa1 -N "" -q -f $cloned_mnt_pt/etc/ssh/ssh_host_key

# Remove any old entry, then copy clone's public key to known_hosts file
  echo "Adding $target_IP to known_hosts file"
  cd /root/.ssh
  if [ -f known_hosts ]; then
    grep -v $target_IP known_hosts > known_hosts.temp
    mv known_hosts.temp known_hosts
  fi

# construct known_hosts entry with three fields: IP@ "ssh-rsa" clones_public_key
  new_key=`cat $cloned_mnt_pt/etc/ssh/ssh_host_rsa_key.pub | awk '{ print $2 }'` 
  echo "$target_IP ssh-rsa $new_key" >> known_hosts

# clean up
  sync # sync disks
  umount $cloned_mnt_pt
  chccwdev -d $target_vaddr
  cp_cmd DETACH $target_vaddr 
  return 0
 }

#+--------------------------------------------------------------------------+
function make_backup_dir()
# Create a directory /backup/<VM-ID>-on-<IP.addr> if it doesn't exist 
# e.g. LINUX04 with IP@ 129.40.178.44 would create /backup/LINUX04-on-129.40.178.44/
#+--------------------------------------------------------------------------+
 {
  if [ -d $backup_dir ]; then # main backup directory exists
    echo "Creating a directory under $backup_dir"
    cd $backup_dir
    backup_subdir=$clone_id-on-$target_IP
    if [ ! -d $backup_subdir ]; then # backup subdirectory does not exist
      mkdir $backup_subdir
      if [ ! $? = 0 ]; then # there was an error from mkdir command
        echo "Error creating directory $backup_dir/$backup_subdir/"
      else
        echo "Created directory $backup_dir/$backup_subdir/"
        echo $descr > $backup_subdir/descLog.txt
      fi
    else # directory exists - create descLog.txt file
      echo $descr > $backup_subdir/descLog.txt
    fi
  fi
 }

# main() # some important global variables
sdisk=0100				   # source disk
clone_id="none"                            # target user ID
was="no"
db2="no"
mqs="no"
umask 002				   # new dir/files set group write bit
cloned_mnt_pt="/mnt/rhel5cloned"           # directory of temporary mount point
backup_dir="/backup/linux"                 # directory of Linux backups

process_arguments $@                       # process arguments passed by user 
if [ $verbose = "on" ]; then set -vx; fi   # turn debug on
check_target_id                            # verify user ID to clone to 
get_conffile_info                          # info about source and target 
ask_are_you_sure                           # confirm disk overwrite
copy_system                                # copy disk $sdisk to target ID
modify_cloned_image $clone_id $sdisk 1100  # modify newly copied system
cp_cmd XAUTOLOG $clone_id                  # bring new clone to life
make_backup_dir                            # make a backup directory
if [ $verbose = "on" ]; then set +vx; fi   # turn debug off
echo "Successfully cloned minidisk $sdisk to $clone_id"
echo "You should be able to ping $target_IP within one minute"
exit 0
