#!/bin/bash
# This script is part of the DAZL Web application
#
# ----------------------------------------------------------------------------
# For details on this code see the books "z/VM and Linux on IBM System z:
# The Virtualization Cookbook for <x>" on the Web at: http://linuxvm.org/present
# where <x> is "SLES 10" or "RHEL 5"
#
# This code is shipped under the Common Public License. See the file
# Common-Public-License.txt or http://www.opensource.org/licenses/cpl1.0.txt
# ----------------------------------------------------------------------------
#
#---------------------------------------------------------------------------
function checkForDir ()
# Be sure /backup/linux/ exists and at least one *-on-* dir
#---------------------------------------------------------------------------
 {
  if [ ! -d $BACKUP ]; then
    echo "Content-type: text/html"
    echo ""
    echo "<h3>Directory $BACKUP not found</h3>"
    exit
  fi
  ls $BACKUP/*-on-* > /dev/null 2>&1 
  if [ $? != 0 ]; then
    echo "Content-type: text/html"
    echo ""
    echo "<h3>No directories of form $BACKUP/*-on-* found</h3>"
    exit
  fi
 }

#---------------------------------------------------------------------------
function backupLinux ()
# Backup /etc/ dirs on all linux systems under /backup/linux/ 
#---------------------------------------------------------------------------
 {
  cd $BACKUP
  for i in *-on-*       # iterate through directories starting with LINUX*-on-*
  do
    IP_addr=${i#*-on-}  # this chops the head off and grabs the IP address
    cd $BACKUP/$i   # change directory
    rsync -r --timeout=1 $IP_addr:/etc . >/dev/null 2>&1 # back up /etc
  done
 } 

#---------------------------------------------------------------------------
function createHeader ()
# Create the page header
#---------------------------------------------------------------------------
 {
  echo "Content-type: text/html"
  echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
  echo '<html xmlns="http://www.w3.org/1999/xhtml">'
  echo ""
  echo "<head>"
  echo '<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />'
  echo "<title>DAZL Linux report</title>"
  echo '<script type="text/javascript" src="../xhr.js"></script>'
  echo '<script type="text/javascript" src="../editLog.js"></script>'
  echo "</head>"
 } # createHeader()

#---------------------------------------------------------------------------
function createBody ()
# Create the page's table by iterating through directories of the form
# /backup/linux/*-on-*/ where the first token is the VM user ID and the
# second token is the Linux IP address
#---------------------------------------------------------------------------
 {
  echo "<body>"
  echo "<table border=2>"
  echo "<tr>"
  echo "<th>z/VM User ID</th>"
  echo "<th>Linux Host name</th>"
  echo "<th>IP address</th>"
  echo "<th colspan=2>Sitar data</th>"
  echo "<th>Description/Log</th>"
  echo "</tr>"
  let j=1
  cd /backup/linux
  if [ $? != 0 ]; then
    echo "Error: directory /backup/linux not found"
    return
  fi
  for i in *-on-*      # iterate through directories "*-on-*" in their name
  do
    if [ $? = '*-on-*' ]; then
      echo "Error: no directories of the form /backup/linux/*-on-* found"
      return
    fi
    # split out the VM user ID and the IP address
    IPaddr=${i#*-on-}  # chop head off and grab IP address
    userID=${i%-on-*}  # chop tail off and grab user ID

    # see if the virtual server is alive
    ping -c1 -w1 $IPaddr > /dev/null 2>&1
    if [ $? = 0 ]; then     # ping was successful 
      let hostIsAlive=1
    else 
      let hostIsAlive=0
    fi

    # start a new row and create the User ID entry
    echo "<tr>"             # start a new table row
    echo "<td>"
    echo "<a style=color:green href=/cgi-bin/getVMdata.sh?$userID>$userID</a>"
    echo "</td>"

    # create the Host Name entry
    if [ -f /backup/vm/$userID.PARMFILE ]; then # SLES 9 PARMFILE is found
      distro="S9"
    elif [ -f /backup/vm/$userID.PARM-S10 ]; then # SLES 10 PARMFILE is found
      distro="S10"
    elif [ -f /backup/vm/$userID.CONF-RH5 ]; then # RHEL 5 PARMFILE is found
      distro="RH5"
    else # no parameter file found
      IP_HOST="parameter file not found"
    fi 
    if [ $distro = "S9" ]; then
      hostLine=`grep IP_HOST /backup/vm/$userID.PARMFILE`
      if [ $? != 0 ]; then # IP_HOST variable not found
        IP_HOST="IP_HOST var not found"
      else
        export local $IP_HOSTline 
      fi
    elif [ $distro = "S10" ]; then 
      hostLine=`grep Hostname /backup/vm/$userID.PARM-S10`
      if [ $? != 0 ]; then # Hostname variable not found
        IP_HOST="Hostname var not found"
      else
        export local $hostLine
        IP_HOST="$Hostname"
      fi
    elif [ $distro = "RH5" ]; then 
      hostLine=`grep HOSTNAME /backup/vm/$userID.CONF-RH5`
      if [ $? != 0 ]; then # HOSTNAME variable not found
        IP_HOST="Hostname var not found"
      else
        export local $hostLine
        IP_HOST="$HOSTNAME"
      fi
    fi
    if [ $hostIsAlive = 1 ]; then # make entry a link to return Linux data
      echo "<td>"
      echo "<a style=color:green href=/cgi-bin/getLinuxData.sh?$IPaddr>"
      echo $IP_HOST
      echo "</a>"
      echo "</td>"
    else # host is dead - can't drill down 
      echo "<td style=color:red>$IP_HOST</td>"
    fi

    # create the IP address entry
    if [ $hostIsAlive = 1 ]; then # make entry a link to return Linux data
      echo "<td>"
      echo "<a style=color:green href=/cgi-bin/getNetworkData.sh?$IPaddr>"
      echo $IPaddr
      echo "</a>"
      echo "</td>"
    else # host is dead - can't get more info
      echo "<td style=color:red>$IPaddr</td>"
    fi

    # create sitar entry for existing report (cron)
    cd /backup/linux/$i/etc # e.g. /backup/linux/LINUX01-on-129.40.49.1/etc
    if [ -f sitar.html ]; then
      echo "<td>"
      echo "<a style=color:green href=/cgi-bin/catSitar.sh?$i>Cron</a>"
      echo "</td>"
    else
      echo "<td style=color:red>"
      echo "Cron"
      echo "</td>"
    fi

    # create sitar entry so user can create a new report
    if [ $hostIsAlive = 1 ]; then # make entry a link to return sitar data
      echo "<td>"
      echo "<a style=color:green href=/cgi-bin/runSitar.sh?$IPaddr>New</a>"
      echo "</td>"
    else
      echo "<td>"
      echo "<a style=color:red>New</a>"
      echo "</td>"
    fi

    # create Description/Log entry
    logFile=/backup/linux/$i/descLog.txt  # e.g. /backup/linux/LINUX01-on-129.40.49.1 
    if [ -f $logFile ]; then
      echo "<td>"
      echo "<div id=${userID}div>"
      echo "<textarea rows=1 cols=70 id=${userID} onClick=EditLog.getLog(this)>"
      head -1 $logFile
      echo "</textarea>"
      echo "<input type=hidden id=${userID}file value=$logFile>"
      echo "</div>"
      echo "</td>"
    else
      echo "<td style=color:red>descLog.txt file not found</td>"
    fi

    # end row and iterate counter
    echo "</tr>"
    let j=$j+1
  done # for each /backup/linux directory
 }

#---------------------------------------------------------------------------
function createFooter()
# end the table and body
#---------------------------------------------------------------------------
 {
  echo "</table>"
  echo "</body>"
 }
  
# main()
BACKUP="/backup/linux"
checkForDir
if [ $CONTENT_LENGTH != 0 ]; then # the one checkbox was set
  read args
  if [ $args = "refreshBackup=on" ]; then
    backupLinux
  fi
fi
createHeader
createBody
createFooter
